package com.qmth.cqb.genpaper.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.cqb.genpaper.context.DetailContext;
import com.qmth.cqb.genpaper.context.PaperContext;
import com.qmth.cqb.genpaper.context.UnitContext;
import com.qmth.cqb.genpaper.model.GenPaperDto;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dao.PaperStructRepo;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.PaperType;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;

@Service
public class GenPaperService {

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo unitRepo;

    @Autowired
    PaperStructRepo paperStructRepo;

    /**
     * 精确组卷，根据设定试卷结构组卷
     * 
     * @return
     */
    @SuppressWarnings("unchecked")
    public Map<String, Object> genPaper(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        LinkedList<Question> questions = new LinkedList<Question>();
        PaperStruct paperStruct = paperStructRepo.findOne(genPaperDto.getPaperStructId());
        PaperContext paperContext = new PaperContext(paperStruct, genPaperDto.getConditions());
        List<UnitContext> unitContexts = new LinkedList<UnitContext>();
        List<DetailContext> detailContexts = paperContext.getDetails();
        for (DetailContext dc : detailContexts) {
            unitContexts.addAll(dc.getUnits());
        }
        List<Paper> papers = (List<Paper>) paperRepo.findAll(genPaperDto.getPaperIds());
        for (Paper oldPaper : papers) {
            List<PaperDetailUnit> unitList = unitRepo.findByPaper(oldPaper);
            for (PaperDetailUnit unit : unitList) {
                questions.add(unit.getQuestion());
            }

        }
        Collections.shuffle(questions);
        int maxLoopCount = questions.size() * 2;
        int selectCount = 0;
        int maxCount = unitContexts.size();
        int loopCount = 0;
        while (questions.size() > 0 && (selectCount < maxCount) && loopCount < maxLoopCount) {
            Question question = questions.removeFirst();
            boolean structTypeMatch = false;
            boolean selected = false;
            for (UnitContext uc : unitContexts) {
                if (uc.getUnitStruct().getQuestionType() == question.getQuestionType() && !uc.finish()) {
                    structTypeMatch = true;
                    if (uc.check(question)) {
                        uc.select(question);
                        selected = true;
                        selectCount++;
                        break;
                    }
                }
            }
            if (structTypeMatch && !selected) {
                questions.addLast(question);
            }
            loopCount++;
        }
        Set<Question> quesSet = paperContext.getSelectQues();
        if (quesSet.size() < unitContexts.size()) {
            msg = "匹配的题源不足，请核查";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }
        List<Object> list = paperContext.build(paperStruct);
        if (list != null && list.size() == 3) {
            Paper paper = (Paper) list.get(0);
            List<PaperDetail> paperDetails = (List<PaperDetail>) list.get(1);
            List<PaperDetailUnit> paperDetailunits = (List<PaperDetailUnit>) list.get(2);
            // 补全paper属性
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            paper.setCreator(genPaperDto.getCreator());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setUnitCount(this.getTotalQuesNum(paperDetailunits));
            // 数据入库
            paper = this.persistentPaper(paperDetailunits, paperDetails, paper);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
        }
        return paperMsgMap;
    }

    /**
     * 按分数在选定的试卷中随机抽取题,组成一套新的试卷
     */
    public Map<String, Object> genPaperByScore(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                Double totalScore = paperMap.get(paperId);
                List<PaperDetailUnit> unitList = unitRepo.findByPaper(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再根据分数取题
                double sum = 0;
                for (PaperDetailUnit pdu : unitList) {
                    sum += pdu.getScore();
                    if (sum <= totalScore) {
                        selectedUnits.add(pdu);
                    } else {
                        break;
                    }
                }
            }
        }
        if (selectedUnits != null && selectedUnits.size() > 0) {
            Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
            msg = "success";
            paperMsgMap.put("paper", paper);
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        } else {
            msg = "没有搜索到相应的试题，请检查题源试卷是否有题目";
            paperMsgMap.put("msg", msg);
            return paperMsgMap;
        }

    }

    /**
     * 按每套试卷需要抽取的题目数量随机抽取题,组成一套新的试卷
     * 
     * @return
     */
    public Map<String, Object> genPaperByQuestionNum(GenPaperDto genPaperDto) {
        String msg = "";
        Map<String, Object> paperMsgMap = new HashMap<String, Object>();
        Map<String, Double> paperMap = genPaperDto.getSimpleParams();
        List<PaperDetailUnit> selectedUnits = new ArrayList<PaperDetailUnit>();// 选中的小题
        int totalQuesNum = 0;
        if (paperMap != null && paperMap.size() > 0) {
            for (String paperId : paperMap.keySet()) {
                int unitNum = paperMap.get(paperId).intValue();
                totalQuesNum += unitNum;
                List<PaperDetailUnit> tempUnits = new ArrayList<PaperDetailUnit>();
                List<PaperDetailUnit> unitList = unitRepo.findByPaper(paperRepo.findOne(paperId));
                Collections.shuffle(unitList);// 随机乱序之后再取题
                if (this.getTotalQuesNum(unitList) >= unitNum) {
                    for (int i = 0; i < unitList.size(); i++) {// 假如unitNum=2，取了第一道题填空题，第二道题是套题，下面有4个子题，就直接进入下一次循环
                        PaperDetailUnit unit = unitList.get(i);
                        int quesNum = this.getTotalQuesNum(tempUnits);
                        if (unit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION
                                && (unit.getQuestion().getSubQuestions().size() + quesNum) > unitNum) {
                            continue;
                        }
                        if (quesNum < unitNum && (!tempUnits.contains(unit))) {
                            tempUnits.add(unit);
                        }
                        if (this.getTotalQuesNum(tempUnits) == unitNum) {
                            break;
                        }

                    }
                    selectedUnits.addAll(tempUnits);
                } else {
                    msg = "题源不足:需要抽取的题目数量要多于试卷中题目数量，请检查";
                    paperMsgMap.put("msg", msg);
                    return paperMsgMap;
                }
            }
            if (this.getTotalQuesNum(selectedUnits) < totalQuesNum) {
                msg = "题源不足:需要抽取的题目数量要多于试卷中题目数量，请检查";
                paperMsgMap.put("msg", msg);
                return paperMsgMap;
            }
        }
        Paper paper = this.constuctPaper(selectedUnits, genPaperDto);
        msg = "success";
        paperMsgMap.put("paper", paper);
        paperMsgMap.put("msg", msg);
        return paperMsgMap;

    }

    /**
     * 构造大题
     * 
     * @param units
     * @param number
     * @param detailName
     *            大题名称
     * @return
     */
    public PaperDetail constuctPaperDeatil(List<PaperDetailUnit> units, int number, String detailName) {
        PaperDetail pd = new PaperDetail();
        if (units != null && units.size() > 0) {
            Double score = 0d;
            for (PaperDetailUnit pdu : units) {
                score += pdu.getScore() == null ? 0 : pdu.getScore();

            }
            pd.setId(UUID.randomUUID().toString());
            pd.setName(detailName);
            pd.setUnitCount(units.size());
            pd.setScore(score);
            pd.setNumber(number);
            pd.setCreateTime(CommonUtils.getCurDateTime());
            for (PaperDetailUnit pdu : units) {
                pdu.setPaperDetail(pd);
            }
        }
        return pd;

    }

    /**
     * 根据大题集合构造paper
     * 
     * @param details
     * @return
     */
    public Paper constuctPaperByPaperDetails(List<PaperDetail> details, GenPaperDto genPaperDto) {
        Paper paper = new Paper();
        if (details != null && details.size() > 0) {
            Double score = 0d;
            for (PaperDetail pd : details) {
                score += pd.getScore() == null ? 0 : pd.getScore();

            }
            paper.setId(UUID.randomUUID().toString());
            paper.setName(genPaperDto.getPaperName());
            paper.setCourseNo(genPaperDto.getCourseNo());
            paper.setCourseName(genPaperDto.getCourseName());
            paper.setCreator(genPaperDto.getCreator());
            paper.setCreateTime(CommonUtils.getCurDateTime());
            paper.setPaperDetailCount(details.size());
            paper.setTotalScore(score);
            paper.setPaperType(PaperType.GENERATE);
        }
        return paper;

    }

    /**
     * 随机构造试卷,根据小题类型 没有定义试卷结构
     * 
     * @param selectedUnits
     * @param genPaperDto
     * @return
     */
    public Paper constuctPaper(List<PaperDetailUnit> selectedUnits, GenPaperDto genPaperDto) {
        Collections.sort(selectedUnits);// 根据大题题型ID排序
        List<QuesStructType> qsts = new ArrayList<QuesStructType>();
        for (int i = 0; i < selectedUnits.size(); i++) {
            if (!qsts.contains(selectedUnits.get(i).getQuestionType())) {
                qsts.add(selectedUnits.get(i).getQuestionType());
            }

        }
        Map<QuesStructType, List<PaperDetailUnit>> unitMap = new HashMap<QuesStructType, List<PaperDetailUnit>>();
        Map<QuesStructType, Integer> qTypeIndexMap = new HashMap<QuesStructType, Integer>();
        for (int i = 1; i <= qsts.size(); i++) {
            List<PaperDetailUnit> tempUnits = new ArrayList<PaperDetailUnit>();
            for (int j = 0; j < selectedUnits.size(); j++) {
                if (qsts.get(i - 1) == selectedUnits.get(j).getQuestionType()) {
                    tempUnits.add(selectedUnits.get(j));
                }
            }
            unitMap.put(qsts.get(i - 1), tempUnits);
            qTypeIndexMap.put(qsts.get(i - 1), i);

        }
        List<PaperDetail> details = new ArrayList<PaperDetail>();
        for (QuesStructType qType : unitMap.keySet()) {
            details.add(this.constuctPaperDeatil(unitMap.get(qType), qTypeIndexMap.get(qType), qType.getName()));
        }
        Paper paper = this.constuctPaperByPaperDetails(details, genPaperDto);
        paper.setUnitCount(this.getTotalQuesNum(selectedUnits));// 设置小题数量，需统计subQuestion
        paper = this.persistentPaper(selectedUnits, details, paper);
        return paper;

    }

    /**
     * 数据入库
     * 
     * @param details
     * @param selectedUnits
     * @param paper
     * @return
     */
    public Paper persistentPaper(List<PaperDetailUnit> selectedUnits, List<PaperDetail> details, Paper paper) {
        paper = paperRepo.save(paper);
        for (PaperDetail pd : details) {
            pd.setPaper(paper);
        }
        paperDetailRepo.save(details);
        for (PaperDetailUnit pdu : selectedUnits) {
            pdu.setPaper(paper);
        }
        unitRepo.save(selectedUnits);
        return paper;

    }

    /**
     * 获取所试题数量
     * 
     * @param units
     * @return
     */
    public int getTotalQuesNum(List<PaperDetailUnit> units) {
        int totalQuesNum = units.size();
        int nestQuesNum = 0;
        for (PaperDetailUnit unit : units) {
            if (unit.getQuestionType() == QuesStructType.NESTED_ANSWER_QUESTION) {
                nestQuesNum += unit.getQuestion().getSubQuestions().size() - 1;
            }
        }
        return totalQuesNum + nestQuesNum;
    }

}
