package com.qmth.cqb.paper.service;

import com.google.gson.Gson;
import com.qmth.cqb.paper.dao.PaperDetailRepo;
import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.Paper;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.utils.BeanCopierUtil;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.word.DocxProcessUtil;
import org.docx4j.XmlUtils;
import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * Created by songyue on 17/3/15.
 */
@Service
public class ExportPaperService {

    @Autowired
    PaperRepo paperRepo;

    @Autowired
    PaperDetailRepo paperDetailRepo;

    @Autowired
    PaperDetailUnitRepo paperDetailUnitRepo;

    @Autowired
    QuesRepo quesRepo;

    @Autowired
    Gson gson;

    @Autowired
    PaperDetailUnitService paperDetailUnitService;

    /**
     * 初始化导出试卷DTO
     * @param id
     * @return
     */
    public Map initExportPaper(String id)throws Exception{
        //创建返回Map
        Map returnMap = new HashMap();
        //获取paper
        Paper paper = paperRepo.findOne(id);
        if(paper == null){
            returnMap.put("errorInfo","该试卷不存在");
            return returnMap;
        }
        //创建paperDto
        PaperExp paperExp = BeanCopierUtil.copyProperties(paper,PaperExp.class);
        //获取大题
        List<PaperDetail> paperDetails = paperDetailRepo.findByPaper(paper);
        List<PaperDetailExp> paperDetailExps =
                BeanCopierUtil.copyPropertiesOfList(paperDetails,PaperDetailExp.class);
        paperExp.setPaperDetails(paperDetailExps);

        //封装小题
        for(int i = 0; i < paperDetailExps.size(); i++){
            List<PaperDetailUnit> paperDetailUnits =
                    paperDetailUnitRepo.findByPaperDetail(paperDetails.get(i));
            List<PaperDetailUnitExp> paperDetailUnitExps =
                    BeanCopierUtil.copyPropertiesOfList(paperDetailUnits,PaperDetailUnitExp.class);
            paperDetailExps.get(i).setPaperDetailUnits(paperDetailUnitExps);
        }
        //初始化试卷序号
        initPaperNum(paperExp);

        returnMap.put("paper", paperExp);
        returnMap.put("fileName", paperExp.getName());

        return returnMap;
    }

    /**
     * 导出试卷
     * @param id
     */
    public String exportPaper(String id) throws Exception {
        Map dataMap = initExportPaper(id);
        if(dataMap.get("errorInfo")!=null){
            return (String)dataMap.get("errorInfo");
        }
        String fileName = (String)dataMap.get("fileName");
        DocxProcessUtil.exportPaper(dataMap,fileName);
        DocxProcessUtil.processImage(fileName,getPkgList(id));
        return "success";
    }

    /**
     * 下载试卷
     * @param id
     */
    public Map downloadPaper(String id, HttpServletResponse response){
        Map dataMap = null;
        try {
            dataMap = initExportPaper(id);
            if(dataMap.get("errorInfo")!=null){
                return dataMap;
            }
            String fileName = (String)dataMap.get("fileName");
            DocxProcessUtil.exportPaper(dataMap,fileName);
            DocxProcessUtil.processImage(fileName,getPkgList(id));
            DocxProcessUtil.processDownload(fileName,response);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return dataMap;
    }

    /**
     * 初始化序号
     * @param paperExp
     */
    public void initPaperNum(PaperExp paperExp)throws Exception{
        if(paperExp.getPaperDetails() == null || paperExp.getPaperDetails().size() == 0){
            return;
        }
        int mainNum = 0;
        int subNum = 0;
        Collections.sort(paperExp.getPaperDetails());
        for(PaperDetailExp paperDetail: paperExp.getPaperDetails()){
            //大题序号
            paperDetail.setCnNum(CommonUtils.toCHNum(paperDetail.getNumber()));
            paperDetail.setNumber(++mainNum);
            if(paperDetail.getPaperDetailUnits() == null ||
                    paperDetail.getPaperDetailUnits().size() == 0){
                return;
            }
            //小题序号
            for(PaperDetailUnitExp paperDetailUnit:paperDetail.getPaperDetailUnits()){
                List<QuesOption> optionList = paperDetailUnit.getQuestion().getQuesOptions();
                if(optionList != null && optionList.size() > 0){
                    for(QuesOption quesOption:optionList){
                        quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),quesOption.getNumber()));
                    }
                }
                List<Question> subQuesList = paperDetailUnit.getQuestion().getSubQuestions();
                //套题序号
                if(subQuesList != null && subQuesList.size() > 0){
                    for(Question subQues:subQuesList){
                        subQues.setQuesBodyWord(setSubQuesNum(subQues.getQuesBodyWord(),++subNum));
                        List<QuesOption> subOptionList = subQues.getQuesOptions();
                        if(subOptionList != null && subOptionList.size() > 0){
                            for(QuesOption quesOption:subOptionList){
                                quesOption.setOptionBodyWord(setOptionNum(quesOption.getOptionBodyWord(),quesOption.getNumber()));
                            }
                        }
                    }
                }else{
                    Question question = paperDetailUnit.getQuestion();
                    question.setQuesBodyWord(setSubQuesNum(question.getQuesBodyWord(),++subNum));
                }
            }
        }
    }

    /**
     * 获取当前试卷下所有试题WordPkg
     * @param id
     * @return
     */
    public List<WordprocessingMLPackage> getPkgList(String id){
        Paper paper = paperRepo.findOne(id);
        List<WordprocessingMLPackage> wordMLPackages = paperDetailUnitRepo.findByPaper(paper)
                .stream().map(PaperDetailUnit::getQuestion).collect(Collectors.toList())
                .stream().map(Question::getPkgObj).collect(Collectors.toList());
        return wordMLPackages;
    }

    /**
     * 设置题号
     * @param quesBodyWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setSubQuesNum(String quesBodyWordMl,int num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + quesBodyWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        P p = (P)body.getContent().get(0);
        List<Object> pContent = p.getContent();
        R run = new R();
        Text text = new Text();
        text.setValue(num+". ");
        run.getContent().add(text);
        pContent.add(0,run);
        return DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(p));
    }

    /**
     * 设置选项号
     * @param optionWordMl
     * @param num
     * @return
     * @throws Exception
     */
    public String setOptionNum(String optionWordMl,String num) throws Exception {
        String tmpStr = DocxProcessUtil.BODY_HEADER + optionWordMl + DocxProcessUtil.BODY_TAIL;
        Body body = (Body)XmlUtils.unmarshalString(tmpStr,
                Context.jc,Body.class);
        P p = (P)body.getContent().get(0);
        List<Object> pContent = p.getContent();
        R run = new R();
        Text text = new Text();
        text.setValue(num+". ");
        run.getContent().add(text);
        pContent.add(0,run);
        return DocxProcessUtil.formatPWordMl(XmlUtils.marshaltoString(p));
    }


}
