package com.qmth.cqb.paper.service;


import java.util.ArrayList;
import java.util.List;

import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.service.QuesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.stereotype.Service;

import com.qmth.cqb.paper.dao.PaperDetailUnitRepo;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;

@Service
public class PaperDetailUnitService {

	@Autowired
	PaperDetailUnitRepo paperDetailUnitRepo;

	@Autowired
	QuesService quesService;

	@Autowired
	QuesRepo quesRepo;

	/**
	 * 根据Id获得对应的试题对象
	 * @param id
	 * @return
	 */
	public Question getQuestionByPaperDetailUnitId(String id){
		return paperDetailUnitRepo.findOne(id).getQuestion();
	}

    /**
     * 按ID查询小题
     * @param id
     * @return
     */
    public PaperDetailUnit findById(String id){
        return paperDetailUnitRepo.findOne(id);
    }

    /**
     * 保存小题
     * @param pdu
     * @return
     */
    public PaperDetailUnit savePaperDetailUnit(PaperDetailUnit pdu){
        return paperDetailUnitRepo.save(pdu);
    }

    /**
     * 删除小题
     * @param id
     * @return
     */
    public void deletePaperDetailUnit(String id){
        paperDetailUnitRepo.delete(id);
    }

    /**
     * 根据大题查小题
     * @param paperDetail
     * @return
     */
    public List<PaperDetailUnit> getUnitsByPaperDetail(PaperDetail paperDetail){
    	return paperDetailUnitRepo.findByPaperDetail(paperDetail);
    }
    
    /**
     * 根据大题集合删除小题
     * @param paperDetails
     */
    public void deleteUnitsByPaperDetails (List<PaperDetail> paperDetails){
    	List<PaperDetailUnit> units = new ArrayList<PaperDetailUnit>();
    	for(PaperDetail pd:paperDetails){
    		units.addAll(getUnitsByPaperDetail(pd));
    	}
    	paperDetailUnitRepo.delete(units);
    }
}

