package com.qmth.cqb.paper.service;


import com.qmth.cqb.paper.dto.PaperDetailUnitStructDto;
import com.qmth.cqb.paper.model.*;
import com.qmth.cqb.utils.BeanCopierUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import com.qmth.cqb.paper.dao.PaperStructRepo;

import java.util.ArrayList;
import java.util.List;
/**
 * Created by songyue on 16/12/28.
 */
@Service
public class PaperStructService {
	
	@Autowired
    PaperStructRepo paperStructRepo;


    /**
     * 获取所有试卷结构
     * @param searchInfo
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<PaperStruct> getPaperStructs(PaperStructSearchInfo searchInfo,
                                             int curPage,
                                             int pageSize){
        formatSearchInfo(searchInfo);
        PaperStruct paperStruct = BeanCopierUtil.copyProperties(searchInfo,PaperStruct.class);
        formatPaperStruct(paperStruct);
        ExampleMatcher matcher = ExampleMatcher.matching().
                withStringMatcher(ExampleMatcher.StringMatcher.CONTAINING).withIgnoreNullValues();
        return paperStructRepo.findAll(Example.of(paperStruct,matcher),
                new PageRequest(curPage - 1,pageSize));
    }

    public void formatSearchInfo(PaperStructSearchInfo searchInfo){
        if(StringUtils.isEmpty(searchInfo.getName())){
            searchInfo.setName(null);
        }
        if(StringUtils.isEmpty(searchInfo.getCreator())){
            searchInfo.setCreator(null);
        }
    }

    public void formatPaperStruct(PaperStruct paperStruct){
        paperStruct.setCreateTime(null);
    }

    /**
     * 保存试卷结构
     * @param paperStruct
     * @return
     */
    public PaperStruct save(PaperStruct paperStruct){

        List<PaperDetailStruct> paperDetailStructs = paperStruct.getPaperDetailStructs();
        int number = 0;

        for(PaperDetailStruct paperDetailStruct:paperDetailStructs){
            List oldStructs = paperDetailStruct.getPaperDetailUnitStructs();
            if(oldStructs != null && oldStructs.size() > 0){
                oldStructs.clear();
            }
            List<PaperDetailUnitStruct> unitStructs = new ArrayList<PaperDetailUnitStruct>();

            for(PaperDetailUnitStructDto unitStructDto:paperDetailStruct.getUnitStructs()){
                for(int i = 0;i < unitStructDto.getCount();i++){
                    ++number;
                    PaperDetailUnitStruct unitStruct = new PaperDetailUnitStruct();
                    unitStruct.setScore(unitStructDto.getScore());
                    unitStruct.setId(String.valueOf(number));
                    unitStruct.setNumber(number);
                    unitStruct.setQuestionType(unitStructDto.getQuestionType());
                    unitStructs.add(unitStruct);
                }
            }
            paperDetailStruct.setPaperDetailUnitStructs(unitStructs);
        }
        paperStruct.setDetailCount(paperDetailStructs.size());
        return paperStructRepo.save(paperStruct);
    }


}
