package com.qmth.cqb.paper.web;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.cqb.paper.dao.PaperRepo;
import com.qmth.cqb.paper.model.PaperDetail;
import com.qmth.cqb.paper.model.PaperDetailUnit;
import com.qmth.cqb.paper.service.PaperDetailService;
import com.qmth.cqb.paper.service.PaperService;

import io.swagger.annotations.ApiOperation;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class PaperDetailController {

    @Autowired
    PaperDetailService paperDetailService;

    @Autowired
    PaperService paperService;
    
    @Autowired
    PaperRepo paperRepo;

    /**
     * 获取大题对应的小题
     * 
     * @param detail_id
     * @return
     */
    @ApiOperation(value = "获取大题对应的小题", notes = "获取大题对应的小题")
    @GetMapping(value = "/paperDetail/units/{detail_id}")
    public List<PaperDetailUnit> getUnitsByPaperDetailId(String detail_id) {
        return paperDetailService.getUnitsByPaperDetailId(detail_id);
    }

    /**
     * 获取大题
     * 
     * @param detail_id
     * @return
     */
    @ApiOperation(value = "获取大题", notes = "获取大题")
    @GetMapping(value = "/paperDetail/{detail_id}")
    public PaperDetail getPaperDetailId(@PathVariable String detail_id) {
        return paperDetailService.findById(detail_id);
    }

    /**
     * 更新大题信息
     * 
     * @param pd
     * @return
     */
    @ApiOperation(value = "更新试卷中的大题", notes = "更新试卷中的大题")
    @PostMapping(value = "/updatePaperDetail/{paperId}")
    public ResponseEntity updatePaperDetail(@PathVariable String paperId,
                                            @RequestBody PaperDetail pd) {
        pd.setPaper(paperRepo.findOne(paperId));
        PaperDetail paperDetail = paperDetailService.savePaperDetail(pd);
        return new ResponseEntity(paperDetail, HttpStatus.OK);

    }

    /**
     * 新增大题
     * 
     * @param pd
     * @return
     */
    @ApiOperation(value = "新增大题", notes = "新增大题")
    @PostMapping(value = "/paperDetail")
    public ResponseEntity addPaperDetail(@RequestBody PaperDetail pd) {
        PaperDetail paperDetail = paperDetailService.savePaperDetail(pd);
        return new ResponseEntity(paperDetail, HttpStatus.OK);
    }

    /**
     * 删除大题
     * 
     * @param detail_id
     * @return
     */
    @ApiOperation(value = "删除大题", notes = "删除大题")
    @DeleteMapping(value = "/paperDetail/{detail_id}")
    public ResponseEntity removePaperDetail(@PathVariable String detail_id) {
        paperDetailService.deletePaperDetail(detail_id);
        return new ResponseEntity(detail_id, HttpStatus.OK);
    }

    /**
     * 根据试卷ID得到所有大题
     * 
     * @param paperId
     * @return
     */
    @ApiOperation(value = "根据试卷ID得到所有大题", notes = "根据试卷ID得到所有大题")
    @GetMapping(value = "/paperDetail/paper/{paperId}")
    public ResponseEntity getPaperDetailsByPaperId(@PathVariable String paperId) {
        return new ResponseEntity(paperService.findPaperDetailsById(paperId), HttpStatus.OK);
    }
}
