package com.qmth.cqb.paper.web;

import com.qmth.cqb.paper.dao.PaperStructRepo;
import com.qmth.cqb.paper.model.PaperStructSearchInfo;
import com.qmth.cqb.utils.CommonUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import com.qmth.cqb.paper.model.PaperStruct;
import com.qmth.cqb.paper.service.PaperStructService;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Created by songyue on 16/12/28.
 */
@RestController
@RequestMapping("${api_cqb}/")
public class PaperStructController {
	
	@Autowired
    PaperStructService paperStructService;

    @Autowired
    PaperStructRepo paperStructRepo;

    /**
     * 获取所有试卷结构
     * @param
     * @return
     */
    @ApiOperation(value="获取试卷结构带分页",notes="获取试卷结构带分页")
    @GetMapping(value = "/paperStruct/{curPage}/{pageSize}")
    public ResponseEntity getPaperStructs(@ModelAttribute PaperStructSearchInfo searchInfo,
                                                @PathVariable int curPage,
                                                @PathVariable int pageSize){
        return new ResponseEntity(paperStructService.getPaperStructs(searchInfo,curPage,pageSize),
                HttpStatus.OK);
    }

    /**
     * 获取所有试卷结构
     * @param
     * @return
     */
    @ApiOperation(value="获取试卷结构",notes="获取试卷结构")
    @GetMapping(value = "/paperStruct")
    public ResponseEntity findAll(){
        return new ResponseEntity(paperStructRepo.findAll(), HttpStatus.OK);
    }
	
    /**
     * 根据id获取试卷结构
     * @param 
     * @return
     */
    @ApiOperation(value="获取试卷结构",notes="获取试卷结构")
    @GetMapping(value = "/paperStruct/{id}")
    public ResponseEntity getPaperStructById(@PathVariable String id){
        return new ResponseEntity(paperStructRepo.findOne(id),HttpStatus.OK);
    }
    

    /**
     * 更新试卷结构
     * @param 
     * @return
     */
    @ApiOperation(value="更新试卷结构",notes="更新试卷结构")
    @PutMapping(value = "/paperStruct")
    public ResponseEntity updatePaperStruct(@RequestBody PaperStruct ps){
    	PaperStruct paperStruct = paperStructService.save(ps);
    	return new ResponseEntity(paperStruct,HttpStatus.OK);
    }

    /**
     * 新增试卷结构
     * @param ps
     * @return
     */
    @ApiOperation(value="新增试卷结构",notes="新增试卷结构")
    @PostMapping(value = "/paperStruct")
    public ResponseEntity addPaperStruct(@RequestBody PaperStruct ps){
        PaperStruct paperStruct = paperStructService.save(ps);
    	return new ResponseEntity(paperStruct,HttpStatus.OK);
    }

    /**
     * 删除试卷结构
     * @param ids
     * @return
     */
    @ApiOperation(value="删除试卷结构",notes="删除试卷结构")
    @DeleteMapping(value = "/paperStruct/{ids}")
    public ResponseEntity removePaperStruct(@PathVariable String ids){
        List<String> paperList = Stream.of(ids.split(",")).collect(Collectors.toList());
    	paperStructRepo.delete(paperStructRepo.findAll(paperList));
		return new ResponseEntity(HttpStatus.OK);
    }
}
