package com.qmth.cqb.question.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import com.qmth.cqb.question.dao.QuesRepo;
import com.qmth.cqb.question.model.QuesOption;
import com.qmth.cqb.question.model.Question;
import com.qmth.cqb.question.model.QuestionSearchCondition;
import com.qmth.cqb.utils.BeanCopierUtil;

/**
 * Created by songyue on 16/12/28.
 */
@Service
public class QuesService {

    @Autowired
    QuesRepo quesRepo;

    /**
     * 保存试题
     * 
     * @param question
     */
    public void saveQues(Question question) {
        Question saveQues = quesRepo.findOne(question.getId());
        saveQues.setQuesAnswer(question.getQuesAnswer());
        saveQues.setScore(question.getScore());
        saveQues.setQuesBody(question.getQuesBody());
        if (question.getQuesOptions() != null && question.getQuesOptions().size() > 0) {
            List<QuesOption> quesOptions = question.getQuesOptions();
            List<QuesOption> saveQuesOptions = saveQues.getQuesOptions();
            if (saveQuesOptions.size() != quesOptions.size()) {
                saveQues.setQuesOptions(quesOptions);
            } else {
                for (int i = 0; i < quesOptions.size(); i++) {
                    saveQuesOptions.get(i).setOptionBody(quesOptions.get(i).getOptionBody());

                }
            }

        }
        quesRepo.save(saveQues);
    }

    /**
     * 查询所有试题
     * 
     * @param searchCondition
     * @param curPage
     * @param pageSize
     * @return
     */
    public Page<Question> findAll(QuestionSearchCondition searchCondition, int curPage, int pageSize) {
        Question ques = BeanCopierUtil.copyProperties(searchCondition, Question.class);
        Page<Question> list = quesRepo.findAll(Example.of(ques), new PageRequest(curPage - 1, pageSize));
        for (Question question : list) {
            formatQues(question);
        }
        return list;
    }

    /**
     * 按ID获取试题
     * 
     * @param id
     * @return
     */
    public Question findById(String id) {
        Question question = quesRepo.findOne(id);
        formatQues(question);
        return question;
    }

    /**
     * 去除试题中的word
     * 
     * @param question
     */
    public void formatQues(Question question) {
        formatQuesUnit(question);
        if (question.getSubQuestions() != null && question.getSubQuestions().size() > 0) {
            question.getSubQuestions().stream().forEach(subQues -> {
                formatQuesUnit(subQues);
            });
        }
    }

    public void formatQuesUnit(Question question) {
        question.setQuesBodyWord(null);
        question.setQuesAnswerWord(null);
        question.setQuesAnswerAnalysisWord(null);
        question.setQuesPkg(new byte[0]);
        if (question.getQuesOptions() != null && question.getQuesOptions().size() > 0) {
            question.getQuesOptions().stream().forEach(quesOption -> {
                quesOption.setOptionBodyWord(null);
            });
        }
    }

}
